"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from collections import abc as _abc
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
import builtins as _builtins
import sys
import typing as _typing

if sys.version_info >= (3, 10):
    from typing import TypeAlias as _TypeAlias
else:
    from typing_extensions import TypeAlias as _TypeAlias

DESCRIPTOR: _descriptor.FileDescriptor

class _MsgId:
    ValueType = _typing.NewType("ValueType", _builtins.int)
    V: _TypeAlias = ValueType  # noqa: Y015

class _MsgIdEnumTypeWrapper(_enum_type_wrapper._EnumTypeWrapper[_MsgId.ValueType], _builtins.type):
    DESCRIPTOR: _descriptor.EnumDescriptor
    MSG_ID_UNKNOWN: _MsgId.ValueType  # 0
    REQ_DLOPEN: _MsgId.ValueType  # 1
    REQ_DLCLOSE: _MsgId.ValueType  # 2
    REQ_DLSYM: _MsgId.ValueType  # 3
    REQ_PEEK: _MsgId.ValueType  # 4
    REQ_POKE: _MsgId.ValueType  # 5
    REQ_EXEC: _MsgId.ValueType  # 6
    REQ_CALL: _MsgId.ValueType  # 7
    REQ_CLOSE_CLIENT: _MsgId.ValueType  # 8
    REQ_LIST_DIR: _MsgId.ValueType  # 9
    REQ_DUMMY_BLOCK: _MsgId.ValueType  # 10
    REQ_GET_CLASS_LIST: _MsgId.ValueType  # 11
    REQ_SHOW_OBJECT: _MsgId.ValueType  # 12
    REQ_SHOW_CLASS: _MsgId.ValueType  # 13

class MsgId(_MsgId, metaclass=_MsgIdEnumTypeWrapper): ...

MSG_ID_UNKNOWN: MsgId.ValueType  # 0
REQ_DLOPEN: MsgId.ValueType  # 1
REQ_DLCLOSE: MsgId.ValueType  # 2
REQ_DLSYM: MsgId.ValueType  # 3
REQ_PEEK: MsgId.ValueType  # 4
REQ_POKE: MsgId.ValueType  # 5
REQ_EXEC: MsgId.ValueType  # 6
REQ_CALL: MsgId.ValueType  # 7
REQ_CLOSE_CLIENT: MsgId.ValueType  # 8
REQ_LIST_DIR: MsgId.ValueType  # 9
REQ_DUMMY_BLOCK: MsgId.ValueType  # 10
REQ_GET_CLASS_LIST: MsgId.ValueType  # 11
REQ_SHOW_OBJECT: MsgId.ValueType  # 12
REQ_SHOW_CLASS: MsgId.ValueType  # 13
Global___MsgId: _TypeAlias = MsgId  # noqa: Y015

@_typing.final
class RequestShowObject(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestShowObject: _TypeAlias = RequestShowObject  # noqa: Y015

@_typing.final
class ReplyShowObject(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: _builtins.int
    description: _builtins.str
    def __init__(
        self,
        *,
        description: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["description", b"description"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyShowObject: _TypeAlias = ReplyShowObject  # noqa: Y015

@_typing.final
class RequestShowClass(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestShowClass: _TypeAlias = RequestShowClass  # noqa: Y015

@_typing.final
class ReplyShowClass(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: _builtins.int
    description: _builtins.str
    def __init__(
        self,
        *,
        description: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["description", b"description"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyShowClass: _TypeAlias = ReplyShowClass  # noqa: Y015

@_typing.final
class RequestDlopen(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    FILENAME_FIELD_NUMBER: _builtins.int
    MODE_FIELD_NUMBER: _builtins.int
    filename: _builtins.str
    mode: _builtins.int
    def __init__(
        self,
        *,
        filename: _builtins.str = ...,
        mode: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["filename", b"filename", "mode", b"mode"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestDlopen: _TypeAlias = RequestDlopen  # noqa: Y015

@_typing.final
class ReplyDlopen(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    HANDLE_FIELD_NUMBER: _builtins.int
    handle: _builtins.int
    def __init__(
        self,
        *,
        handle: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["handle", b"handle"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDlopen: _TypeAlias = ReplyDlopen  # noqa: Y015

@_typing.final
class RequestDlclose(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    HANDLE_FIELD_NUMBER: _builtins.int
    handle: _builtins.int
    def __init__(
        self,
        *,
        handle: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["handle", b"handle"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestDlclose: _TypeAlias = RequestDlclose  # noqa: Y015

@_typing.final
class ReplyDlclose(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    RES_FIELD_NUMBER: _builtins.int
    res: _builtins.int
    def __init__(
        self,
        *,
        res: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["res", b"res"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDlclose: _TypeAlias = ReplyDlclose  # noqa: Y015

@_typing.final
class RequestDlsym(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    HANDLE_FIELD_NUMBER: _builtins.int
    SYMBOL_NAME_FIELD_NUMBER: _builtins.int
    handle: _builtins.int
    symbol_name: _builtins.str
    def __init__(
        self,
        *,
        handle: _builtins.int = ...,
        symbol_name: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["handle", b"handle", "symbol_name", b"symbol_name"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestDlsym: _TypeAlias = RequestDlsym  # noqa: Y015

@_typing.final
class ReplyDlsym(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PTR_FIELD_NUMBER: _builtins.int
    ptr: _builtins.int
    def __init__(
        self,
        *,
        ptr: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["ptr", b"ptr"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDlsym: _TypeAlias = ReplyDlsym  # noqa: Y015

@_typing.final
class RequestExec(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    BACKGROUND_FIELD_NUMBER: _builtins.int
    ARGV_FIELD_NUMBER: _builtins.int
    ENVP_FIELD_NUMBER: _builtins.int
    background: _builtins.bool
    @_builtins.property
    def argv(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    @_builtins.property
    def envp(self) -> _containers.RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(
        self,
        *,
        background: _builtins.bool = ...,
        argv: _abc.Iterable[_builtins.str] | None = ...,
        envp: _abc.Iterable[_builtins.str] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["argv", b"argv", "background", b"background", "envp", b"envp"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestExec: _TypeAlias = RequestExec  # noqa: Y015

@_typing.final
class ReplyExec(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PID_FIELD_NUMBER: _builtins.int
    FD_FIELD_NUMBER: _builtins.int
    pid: _builtins.int
    fd: _builtins.int
    def __init__(
        self,
        *,
        pid: _builtins.int = ...,
        fd: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["fd", b"fd", "pid", b"pid"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyExec: _TypeAlias = ReplyExec  # noqa: Y015

@_typing.final
class RequestCall(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    VA_LIST_INDEX_FIELD_NUMBER: _builtins.int
    ARGV_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    va_list_index: _builtins.int
    @_builtins.property
    def argv(self) -> _containers.RepeatedCompositeFieldContainer[Global___Argument]: ...
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        va_list_index: _builtins.int = ...,
        argv: _abc.Iterable[Global___Argument] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "argv", b"argv", "va_list_index", b"va_list_index"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestCall: _TypeAlias = RequestCall  # noqa: Y015

@_typing.final
class ReplyCall(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ARM_REGISTERS_FIELD_NUMBER: _builtins.int
    RETURN_VALUE_FIELD_NUMBER: _builtins.int
    return_value: _builtins.int
    @_builtins.property
    def arm_registers(self) -> Global___ReturnRegistersArm: ...
    def __init__(
        self,
        *,
        arm_registers: Global___ReturnRegistersArm | None = ...,
        return_value: _builtins.int = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["arm_registers", b"arm_registers", "return_value", b"return_value", "return_values", b"return_values"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["arm_registers", b"arm_registers", "return_value", b"return_value", "return_values", b"return_values"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_return_values: _TypeAlias = _typing.Literal["arm_registers", "return_value"]  # noqa: Y015
    _WhichOneofArgType_return_values: _TypeAlias = _typing.Literal["return_values", b"return_values"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_return_values) -> _WhichOneofReturnType_return_values | None: ...

Global___ReplyCall: _TypeAlias = ReplyCall  # noqa: Y015

@_typing.final
class RequestPeek(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    SIZE_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    size: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        size: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "size", b"size"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestPeek: _TypeAlias = RequestPeek  # noqa: Y015

@_typing.final
class ReplyPeek(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DATA_FIELD_NUMBER: _builtins.int
    data: _builtins.bytes
    def __init__(
        self,
        *,
        data: _builtins.bytes = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyPeek: _TypeAlias = ReplyPeek  # noqa: Y015

@_typing.final
class RequestPoke(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    DATA_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    data: _builtins.bytes
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        data: _builtins.bytes = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "data", b"data"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestPoke: _TypeAlias = RequestPoke  # noqa: Y015

@_typing.final
class ReplyPoke(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___ReplyPoke: _TypeAlias = ReplyPoke  # noqa: Y015

@_typing.final
class RequestListDir(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    PATH_FIELD_NUMBER: _builtins.int
    path: _builtins.str
    def __init__(
        self,
        *,
        path: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["path", b"path"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___RequestListDir: _TypeAlias = RequestListDir  # noqa: Y015

@_typing.final
class RequestDummyBlock(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___RequestDummyBlock: _TypeAlias = RequestDummyBlock  # noqa: Y015

@_typing.final
class ReplyDummyBlock(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    SIZE_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    size: _builtins.int
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        size: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "size", b"size"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyDummyBlock: _TypeAlias = ReplyDummyBlock  # noqa: Y015

@_typing.final
class RequestGetClassList(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___RequestGetClassList: _TypeAlias = RequestGetClassList  # noqa: Y015

@_typing.final
class ObjcClass(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: _builtins.int
    NAME_FIELD_NUMBER: _builtins.int
    address: _builtins.int
    name: _builtins.str
    def __init__(
        self,
        *,
        address: _builtins.int = ...,
        name: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["address", b"address", "name", b"name"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ObjcClass: _TypeAlias = ObjcClass  # noqa: Y015

@_typing.final
class ReplyGetClassList(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    CLASSES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def classes(self) -> _containers.RepeatedCompositeFieldContainer[Global___ObjcClass]: ...
    def __init__(
        self,
        *,
        classes: _abc.Iterable[Global___ObjcClass] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["classes", b"classes"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyGetClassList: _TypeAlias = ReplyGetClassList  # noqa: Y015

@_typing.final
class ReplyError(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: _builtins.int
    message: _builtins.str
    def __init__(
        self,
        *,
        message: _builtins.str = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["message", b"message"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyError: _TypeAlias = ReplyError  # noqa: Y015

@_typing.final
class ReplyListDir(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    DIR_ENTRIES_FIELD_NUMBER: _builtins.int
    @_builtins.property
    def dir_entries(self) -> _containers.RepeatedCompositeFieldContainer[Global___DirEntry]: ...
    def __init__(
        self,
        *,
        dir_entries: _abc.Iterable[Global___DirEntry] | None = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["dir_entries", b"dir_entries"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReplyListDir: _TypeAlias = ReplyListDir  # noqa: Y015

@_typing.final
class RequestCloseClient(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___RequestCloseClient: _TypeAlias = RequestCloseClient  # noqa: Y015

@_typing.final
class ReplyCloseClient(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

Global___ReplyCloseClient: _TypeAlias = ReplyCloseClient  # noqa: Y015

@_typing.final
class DirEntryStat(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    ERRNO1_FIELD_NUMBER: _builtins.int
    ST_DEV_FIELD_NUMBER: _builtins.int
    ST_MODE_FIELD_NUMBER: _builtins.int
    ST_NLINK_FIELD_NUMBER: _builtins.int
    ST_INO_FIELD_NUMBER: _builtins.int
    ST_UID_FIELD_NUMBER: _builtins.int
    ST_GID_FIELD_NUMBER: _builtins.int
    ST_RDEV_FIELD_NUMBER: _builtins.int
    ST_SIZE_FIELD_NUMBER: _builtins.int
    ST_BLOCKS_FIELD_NUMBER: _builtins.int
    ST_BLKSIZE_FIELD_NUMBER: _builtins.int
    ST_ATIME1_FIELD_NUMBER: _builtins.int
    ST_MTIME1_FIELD_NUMBER: _builtins.int
    ST_CTIME1_FIELD_NUMBER: _builtins.int
    errno1: _builtins.int
    st_dev: _builtins.int
    st_mode: _builtins.int
    st_nlink: _builtins.int
    st_ino: _builtins.int
    st_uid: _builtins.int
    st_gid: _builtins.int
    st_rdev: _builtins.int
    st_size: _builtins.int
    st_blocks: _builtins.int
    st_blksize: _builtins.int
    st_atime1: _builtins.int
    st_mtime1: _builtins.int
    st_ctime1: _builtins.int
    def __init__(
        self,
        *,
        errno1: _builtins.int = ...,
        st_dev: _builtins.int = ...,
        st_mode: _builtins.int = ...,
        st_nlink: _builtins.int = ...,
        st_ino: _builtins.int = ...,
        st_uid: _builtins.int = ...,
        st_gid: _builtins.int = ...,
        st_rdev: _builtins.int = ...,
        st_size: _builtins.int = ...,
        st_blocks: _builtins.int = ...,
        st_blksize: _builtins.int = ...,
        st_atime1: _builtins.int = ...,
        st_mtime1: _builtins.int = ...,
        st_ctime1: _builtins.int = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["errno1", b"errno1", "st_atime1", b"st_atime1", "st_blksize", b"st_blksize", "st_blocks", b"st_blocks", "st_ctime1", b"st_ctime1", "st_dev", b"st_dev", "st_gid", b"st_gid", "st_ino", b"st_ino", "st_mode", b"st_mode", "st_mtime1", b"st_mtime1", "st_nlink", b"st_nlink", "st_rdev", b"st_rdev", "st_size", b"st_size", "st_uid", b"st_uid"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DirEntryStat: _TypeAlias = DirEntryStat  # noqa: Y015

@_typing.final
class DirEntry(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    D_TYPE_FIELD_NUMBER: _builtins.int
    D_NAME_FIELD_NUMBER: _builtins.int
    LSTAT_FIELD_NUMBER: _builtins.int
    STAT_FIELD_NUMBER: _builtins.int
    d_type: _builtins.int
    d_name: _builtins.str
    @_builtins.property
    def lstat(self) -> Global___DirEntryStat: ...
    @_builtins.property
    def stat(self) -> Global___DirEntryStat: ...
    def __init__(
        self,
        *,
        d_type: _builtins.int = ...,
        d_name: _builtins.str = ...,
        lstat: Global___DirEntryStat | None = ...,
        stat: Global___DirEntryStat | None = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["lstat", b"lstat", "stat", b"stat"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["d_name", b"d_name", "d_type", b"d_type", "lstat", b"lstat", "stat", b"stat"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___DirEntry: _TypeAlias = DirEntry  # noqa: Y015

@_typing.final
class ReturnRegistersArm(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    X0_FIELD_NUMBER: _builtins.int
    X1_FIELD_NUMBER: _builtins.int
    X2_FIELD_NUMBER: _builtins.int
    X3_FIELD_NUMBER: _builtins.int
    X4_FIELD_NUMBER: _builtins.int
    X5_FIELD_NUMBER: _builtins.int
    X6_FIELD_NUMBER: _builtins.int
    X7_FIELD_NUMBER: _builtins.int
    D0_FIELD_NUMBER: _builtins.int
    D1_FIELD_NUMBER: _builtins.int
    D2_FIELD_NUMBER: _builtins.int
    D3_FIELD_NUMBER: _builtins.int
    D4_FIELD_NUMBER: _builtins.int
    D5_FIELD_NUMBER: _builtins.int
    D6_FIELD_NUMBER: _builtins.int
    D7_FIELD_NUMBER: _builtins.int
    x0: _builtins.int
    x1: _builtins.int
    x2: _builtins.int
    x3: _builtins.int
    x4: _builtins.int
    x5: _builtins.int
    x6: _builtins.int
    x7: _builtins.int
    d0: _builtins.float
    d1: _builtins.float
    d2: _builtins.float
    d3: _builtins.float
    d4: _builtins.float
    d5: _builtins.float
    d6: _builtins.float
    d7: _builtins.float
    def __init__(
        self,
        *,
        x0: _builtins.int = ...,
        x1: _builtins.int = ...,
        x2: _builtins.int = ...,
        x3: _builtins.int = ...,
        x4: _builtins.int = ...,
        x5: _builtins.int = ...,
        x6: _builtins.int = ...,
        x7: _builtins.int = ...,
        d0: _builtins.float = ...,
        d1: _builtins.float = ...,
        d2: _builtins.float = ...,
        d3: _builtins.float = ...,
        d4: _builtins.float = ...,
        d5: _builtins.float = ...,
        d6: _builtins.float = ...,
        d7: _builtins.float = ...,
    ) -> None: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["d0", b"d0", "d1", b"d1", "d2", b"d2", "d3", b"d3", "d4", b"d4", "d5", b"d5", "d6", b"d6", "d7", b"d7", "x0", b"x0", "x1", b"x1", "x2", b"x2", "x3", b"x3", "x4", b"x4", "x5", b"x5", "x6", b"x6", "x7", b"x7"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...

Global___ReturnRegistersArm: _TypeAlias = ReturnRegistersArm  # noqa: Y015

@_typing.final
class Argument(_message.Message):
    DESCRIPTOR: _descriptor.Descriptor

    V_INT_FIELD_NUMBER: _builtins.int
    V_DOUBLE_FIELD_NUMBER: _builtins.int
    V_STR_FIELD_NUMBER: _builtins.int
    V_BYTES_FIELD_NUMBER: _builtins.int
    v_int: _builtins.int
    v_double: _builtins.float
    v_str: _builtins.str
    v_bytes: _builtins.bytes
    def __init__(
        self,
        *,
        v_int: _builtins.int = ...,
        v_double: _builtins.float = ...,
        v_str: _builtins.str = ...,
        v_bytes: _builtins.bytes = ...,
    ) -> None: ...
    _HasFieldArgType: _TypeAlias = _typing.Literal["type", b"type", "v_bytes", b"v_bytes", "v_double", b"v_double", "v_int", b"v_int", "v_str", b"v_str"]  # noqa: Y015
    def HasField(self, field_name: _HasFieldArgType) -> _builtins.bool: ...
    _ClearFieldArgType: _TypeAlias = _typing.Literal["type", b"type", "v_bytes", b"v_bytes", "v_double", b"v_double", "v_int", b"v_int", "v_str", b"v_str"]  # noqa: Y015
    def ClearField(self, field_name: _ClearFieldArgType) -> None: ...
    _WhichOneofReturnType_type: _TypeAlias = _typing.Literal["v_int", "v_double", "v_str", "v_bytes"]  # noqa: Y015
    _WhichOneofArgType_type: _TypeAlias = _typing.Literal["type", b"type"]  # noqa: Y015
    def WhichOneof(self, oneof_group: _WhichOneofArgType_type) -> _WhichOneofReturnType_type | None: ...

Global___Argument: _TypeAlias = Argument  # noqa: Y015
