# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: rpc_api.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'rpc_api.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rrpc_api.proto\x12\x07rpc.api\"$\n\x11RequestShowObject\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\"&\n\x0fReplyShowObject\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\"#\n\x10RequestShowClass\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\"%\n\x0eReplyShowClass\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\"/\n\rRequestDlopen\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0c\n\x04mode\x18\x02 \x01(\x05\"\x1d\n\x0bReplyDlopen\x12\x0e\n\x06handle\x18\x01 \x01(\x04\" \n\x0eRequestDlclose\x12\x0e\n\x06handle\x18\x01 \x01(\x04\"\x1b\n\x0cReplyDlclose\x12\x0b\n\x03res\x18\x01 \x01(\r\"3\n\x0cRequestDlsym\x12\x0e\n\x06handle\x18\x01 \x01(\x04\x12\x13\n\x0bsymbol_name\x18\x02 \x01(\t\"\x19\n\nReplyDlsym\x12\x0b\n\x03ptr\x18\x01 \x01(\x04\"=\n\x0bRequestExec\x12\x12\n\nbackground\x18\x01 \x01(\x08\x12\x0c\n\x04\x61rgv\x18\x02 \x03(\t\x12\x0c\n\x04\x65nvp\x18\x03 \x03(\t\"$\n\tReplyExec\x12\x0b\n\x03pid\x18\x01 \x01(\r\x12\n\n\x02\x66\x64\x18\x02 \x01(\x05\"V\n\x0bRequestCall\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x15\n\rva_list_index\x18\x02 \x01(\x04\x12\x1f\n\x04\x61rgv\x18\x03 \x03(\x0b\x32\x11.rpc.api.Argument\"j\n\tReplyCall\x12\x34\n\rarm_registers\x18\x01 \x01(\x0b\x32\x1b.rpc.api.ReturnRegistersArmH\x00\x12\x16\n\x0creturn_value\x18\x02 \x01(\x04H\x00\x42\x0f\n\rreturn_values\",\n\x0bRequestPeek\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04size\x18\x02 \x01(\x04\"\x19\n\tReplyPeek\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\",\n\x0bRequestPoke\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\x0b\n\tReplyPoke\"\x1e\n\x0eRequestListDir\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x13\n\x11RequestDummyBlock\"0\n\x0fReplyDummyBlock\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04size\x18\x02 \x01(\x04\"\x15\n\x13RequestGetClassList\"*\n\tObjcClass\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x04\x12\x0c\n\x04name\x18\x02 \x01(\t\"8\n\x11ReplyGetClassList\x12#\n\x07\x63lasses\x18\x01 \x03(\x0b\x32\x12.rpc.api.ObjcClass\"\x1d\n\nReplyError\x12\x0f\n\x07message\x18\x01 \x01(\t\"6\n\x0cReplyListDir\x12&\n\x0b\x64ir_entries\x18\x03 \x03(\x0b\x32\x11.rpc.api.DirEntry\"\x14\n\x12RequestCloseClient\"\x12\n\x10ReplyCloseClient\"\x83\x02\n\x0c\x44irEntryStat\x12\x0e\n\x06\x65rrno1\x18\x01 \x01(\x04\x12\x0e\n\x06st_dev\x18\x02 \x01(\x04\x12\x0f\n\x07st_mode\x18\x03 \x01(\x04\x12\x10\n\x08st_nlink\x18\x04 \x01(\x04\x12\x0e\n\x06st_ino\x18\x05 \x01(\x04\x12\x0e\n\x06st_uid\x18\x06 \x01(\x04\x12\x0e\n\x06st_gid\x18\x07 \x01(\x04\x12\x0f\n\x07st_rdev\x18\x08 \x01(\x04\x12\x0f\n\x07st_size\x18\t \x01(\x04\x12\x11\n\tst_blocks\x18\n \x01(\x04\x12\x12\n\nst_blksize\x18\x0b \x01(\x04\x12\x11\n\tst_atime1\x18\x0c \x01(\x04\x12\x11\n\tst_mtime1\x18\r \x01(\x04\x12\x11\n\tst_ctime1\x18\x0e \x01(\x04\"u\n\x08\x44irEntry\x12\x0e\n\x06\x64_type\x18\x01 \x01(\x04\x12\x0e\n\x06\x64_name\x18\x02 \x01(\t\x12$\n\x05lstat\x18\x03 \x01(\x0b\x32\x15.rpc.api.DirEntryStat\x12#\n\x04stat\x18\x04 \x01(\x0b\x32\x15.rpc.api.DirEntryStat\"\xd4\x01\n\x12ReturnRegistersArm\x12\n\n\x02x0\x18\x01 \x01(\x04\x12\n\n\x02x1\x18\x02 \x01(\x04\x12\n\n\x02x2\x18\x03 \x01(\x04\x12\n\n\x02x3\x18\x04 \x01(\x04\x12\n\n\x02x4\x18\x05 \x01(\x04\x12\n\n\x02x5\x18\x06 \x01(\x04\x12\n\n\x02x6\x18\x07 \x01(\x04\x12\n\n\x02x7\x18\x08 \x01(\x04\x12\n\n\x02\x64\x30\x18\t \x01(\x01\x12\n\n\x02\x64\x31\x18\n \x01(\x01\x12\n\n\x02\x64\x32\x18\x0b \x01(\x01\x12\n\n\x02\x64\x33\x18\x0c \x01(\x01\x12\n\n\x02\x64\x34\x18\r \x01(\x01\x12\n\n\x02\x64\x35\x18\x0e \x01(\x01\x12\n\n\x02\x64\x36\x18\x0f \x01(\x01\x12\n\n\x02\x64\x37\x18\x10 \x01(\x01\"[\n\x08\x41rgument\x12\x0f\n\x05v_int\x18\x01 \x01(\x04H\x00\x12\x12\n\x08v_double\x18\x02 \x01(\x01H\x00\x12\x0f\n\x05v_str\x18\x03 \x01(\tH\x00\x12\x11\n\x07v_bytes\x18\x04 \x01(\x0cH\x00\x42\x06\n\x04type*\x8c\x02\n\x05MsgId\x12\x12\n\x0eMSG_ID_UNKNOWN\x10\x00\x12\x0e\n\nREQ_DLOPEN\x10\x01\x12\x0f\n\x0bREQ_DLCLOSE\x10\x02\x12\r\n\tREQ_DLSYM\x10\x03\x12\x0c\n\x08REQ_PEEK\x10\x04\x12\x0c\n\x08REQ_POKE\x10\x05\x12\x0c\n\x08REQ_EXEC\x10\x06\x12\x0c\n\x08REQ_CALL\x10\x07\x12\x14\n\x10REQ_CLOSE_CLIENT\x10\x08\x12\x10\n\x0cREQ_LIST_DIR\x10\t\x12\x13\n\x0fREQ_DUMMY_BLOCK\x10\n\x12\x16\n\x12REQ_GET_CLASS_LIST\x10\x0b\x12\x13\n\x0fREQ_SHOW_OBJECT\x10\x0c\x12\x12\n\x0eREQ_SHOW_CLASS\x10\r\"\t\x08\x80\x02\x10\xff\xff\xff\xff\x07\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rpc_api_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MSGID']._serialized_start=1879
  _globals['_MSGID']._serialized_end=2147
  _globals['_REQUESTSHOWOBJECT']._serialized_start=26
  _globals['_REQUESTSHOWOBJECT']._serialized_end=62
  _globals['_REPLYSHOWOBJECT']._serialized_start=64
  _globals['_REPLYSHOWOBJECT']._serialized_end=102
  _globals['_REQUESTSHOWCLASS']._serialized_start=104
  _globals['_REQUESTSHOWCLASS']._serialized_end=139
  _globals['_REPLYSHOWCLASS']._serialized_start=141
  _globals['_REPLYSHOWCLASS']._serialized_end=178
  _globals['_REQUESTDLOPEN']._serialized_start=180
  _globals['_REQUESTDLOPEN']._serialized_end=227
  _globals['_REPLYDLOPEN']._serialized_start=229
  _globals['_REPLYDLOPEN']._serialized_end=258
  _globals['_REQUESTDLCLOSE']._serialized_start=260
  _globals['_REQUESTDLCLOSE']._serialized_end=292
  _globals['_REPLYDLCLOSE']._serialized_start=294
  _globals['_REPLYDLCLOSE']._serialized_end=321
  _globals['_REQUESTDLSYM']._serialized_start=323
  _globals['_REQUESTDLSYM']._serialized_end=374
  _globals['_REPLYDLSYM']._serialized_start=376
  _globals['_REPLYDLSYM']._serialized_end=401
  _globals['_REQUESTEXEC']._serialized_start=403
  _globals['_REQUESTEXEC']._serialized_end=464
  _globals['_REPLYEXEC']._serialized_start=466
  _globals['_REPLYEXEC']._serialized_end=502
  _globals['_REQUESTCALL']._serialized_start=504
  _globals['_REQUESTCALL']._serialized_end=590
  _globals['_REPLYCALL']._serialized_start=592
  _globals['_REPLYCALL']._serialized_end=698
  _globals['_REQUESTPEEK']._serialized_start=700
  _globals['_REQUESTPEEK']._serialized_end=744
  _globals['_REPLYPEEK']._serialized_start=746
  _globals['_REPLYPEEK']._serialized_end=771
  _globals['_REQUESTPOKE']._serialized_start=773
  _globals['_REQUESTPOKE']._serialized_end=817
  _globals['_REPLYPOKE']._serialized_start=819
  _globals['_REPLYPOKE']._serialized_end=830
  _globals['_REQUESTLISTDIR']._serialized_start=832
  _globals['_REQUESTLISTDIR']._serialized_end=862
  _globals['_REQUESTDUMMYBLOCK']._serialized_start=864
  _globals['_REQUESTDUMMYBLOCK']._serialized_end=883
  _globals['_REPLYDUMMYBLOCK']._serialized_start=885
  _globals['_REPLYDUMMYBLOCK']._serialized_end=933
  _globals['_REQUESTGETCLASSLIST']._serialized_start=935
  _globals['_REQUESTGETCLASSLIST']._serialized_end=956
  _globals['_OBJCCLASS']._serialized_start=958
  _globals['_OBJCCLASS']._serialized_end=1000
  _globals['_REPLYGETCLASSLIST']._serialized_start=1002
  _globals['_REPLYGETCLASSLIST']._serialized_end=1058
  _globals['_REPLYERROR']._serialized_start=1060
  _globals['_REPLYERROR']._serialized_end=1089
  _globals['_REPLYLISTDIR']._serialized_start=1091
  _globals['_REPLYLISTDIR']._serialized_end=1145
  _globals['_REQUESTCLOSECLIENT']._serialized_start=1147
  _globals['_REQUESTCLOSECLIENT']._serialized_end=1167
  _globals['_REPLYCLOSECLIENT']._serialized_start=1169
  _globals['_REPLYCLOSECLIENT']._serialized_end=1187
  _globals['_DIRENTRYSTAT']._serialized_start=1190
  _globals['_DIRENTRYSTAT']._serialized_end=1449
  _globals['_DIRENTRY']._serialized_start=1451
  _globals['_DIRENTRY']._serialized_end=1568
  _globals['_RETURNREGISTERSARM']._serialized_start=1571
  _globals['_RETURNREGISTERSARM']._serialized_end=1783
  _globals['_ARGUMENT']._serialized_start=1785
  _globals['_ARGUMENT']._serialized_end=1876
# @@protoc_insertion_point(module_scope)
