# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: rpc.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'rpc.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\trpc.proto\x12\x03rpc\"O\n\nRpcMessage\x12\r\n\x05magic\x18\x01 \x01(\r\x12\x11\n\tclient_id\x18\x02 \x01(\r\x12\x0e\n\x06msg_id\x18\x03 \x01(\r\x12\x0f\n\x07payload\x18\x04 \x01(\x0c\">\n\rRpcPtyMessage\x12\x10\n\x06\x62uffer\x18\x01 \x01(\x0cH\x00\x12\x13\n\texit_code\x18\x02 \x01(\rH\x00\x42\x06\n\x04type\"\x92\x01\n\tHandshake\x12\r\n\x05magic\x18\x01 \x01(\r\x12\x17\n\x04\x61rch\x18\x02 \x01(\x0e\x32\t.rpc.Arch\x12\x0f\n\x07sysname\x18\x03 \x01(\t\x12\x0f\n\x07machine\x18\x04 \x01(\t\x12\x10\n\x08platform\x18\x05 \x01(\t\x12\x16\n\x0eserver_version\x18\x07 \x01(\r\x12\x11\n\tclient_id\x18\x08 \x01(\r*\x91\x01\n\x11ProtocolConstants\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x15\n\x0eSERVER_VERSION\x10\x91\x90\xa2\x44\x12\x14\n\rMESSAGE_MAGIC\x10\xe9\x8a\x8d\t\x12\x0e\n\tREP_ERROR\x10\x80 \x12\x19\n\x13RPC_PTY_BUFFER_SIZE\x10\x80\x80\x04\x12\x17\n\x12RPC_MAX_REQ_MSG_ID\x10\x80\x02*(\n\x04\x41rch\x12\x10\n\x0c\x41RCH_UNKNOWN\x10\x00\x12\x0e\n\nARCH_ARM64\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rpc_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_PROTOCOLCONSTANTS']._serialized_start=313
  _globals['_PROTOCOLCONSTANTS']._serialized_end=458
  _globals['_ARCH']._serialized_start=460
  _globals['_ARCH']._serialized_end=500
  _globals['_RPCMESSAGE']._serialized_start=18
  _globals['_RPCMESSAGE']._serialized_end=97
  _globals['_RPCPTYMESSAGE']._serialized_start=99
  _globals['_RPCPTYMESSAGE']._serialized_end=161
  _globals['_HANDSHAKE']._serialized_start=164
  _globals['_HANDSHAKE']._serialized_end=310
# @@protoc_insertion_point(module_scope)
